@@ -1,7 +1,7 @@
 /* main.c - the normal mode main routine */
 /*
  *  GRUB  --  GRand Unified Bootloader
- *  Copyright (C) 2000,2001,2002,2003,2005  Free Software Foundation, Inc.
+ *  Copyright (C) 2000,2001,2002,2003,2005,2006  Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -153,6 +153,8 @@ grub_normal_menu_addentry (const char *title, struct grub_script *script,
   (*last)->next = 0;
   (*last)->sourcecode = sourcecode;
 
+  current_menu->size++;
+
   return GRUB_ERR_NONE;
 }
 
@@ -162,6 +164,7 @@ read_config_file (const char *config)
   grub_file_t file;
   auto grub_err_t getline (char **line);
   int currline = 0;
+  int errors = 0;
   
   grub_err_t getline (char **line)
     {
@@ -184,9 +187,6 @@ read_config_file (const char *config)
   newmenu = grub_malloc (sizeof (*newmenu));
   if (! newmenu)
     return 0;
-  newmenu->default_entry = 0;
-  newmenu->fallback_entry = -1;
-  newmenu->timeout = -1;
   newmenu->size = 0;
   newmenu->entry_list = 0;
   current_menu = newmenu;
@@ -199,21 +199,18 @@ read_config_file (const char *config)
   while (get_line (file, cmdline, sizeof (cmdline)))
     {
       struct grub_script *parsed_script;
+      int startline;
       
-      currline++;
+      startline = ++currline;
 
       /* Execute the script, line for line.  */
       parsed_script = grub_script_parse (cmdline, getline);
 
       if (! parsed_script)
 	{
-	  /* Wait until the user pushes any key so that the user can
-	     see what happened.  */
-	  grub_printf ("\nPress any key to continue...");
-	  (void) grub_getkey ();
-
-	  grub_file_close (file);
-	  return 0;
+	  grub_printf ("(line %d-%d)\n", startline, currline);
+	  errors++;
+	  continue;
 	}
 
       /* Execute the command(s).  */
@@ -223,10 +220,24 @@ read_config_file (const char *config)
       grub_script_free (parsed_script);
     }
 
-  return newmenu;
-
   grub_file_close (file);
-  return 0;
+
+  if (errors > 0)
+    {
+      /* Wait until the user pushes any key so that the user can
+	 see what happened.  */
+      grub_printf ("\nPress any key to continue...");
+      (void) grub_getkey ();
+    }
+
+  /* If the menu is empty, just drop it.  */
+  if (current_menu->size == 0)
+    {
+      grub_free (current_menu);
+      return 0;
+    }
+
+  return newmenu;
 }
 
 /* This starts the normal mode.  */
@@ -417,8 +428,9 @@ grub_normal_execute (const char *config, int nested)
 
   if (menu)
     {
+      grub_env_set_data_slot ("menu", menu);
       grub_menu_run (menu, nested);
-      grub_context_pop_menu ();
+      grub_env_unset_data_slot ("menu");
       free_menu (menu);
     }
   else
